﻿Imports System.Runtime.CompilerServices
Imports System.Reflection
'Imports Helpers.BitOperation

Namespace System.ComponentModel
   'NotifyPropertyChanged
   Public Class NotifyPropertyChanged : Implements INotifyPropertyChanged, ICloneable

      Private Shared _IsInDesignMode As Boolean? = Nothing
      Private Shared _HostProcesses As New List(Of String)({"XDesProc", "denev"})
      Public Shared Function IsInDesignMode() As Boolean
         If Not _IsInDesignMode.HasValue Then
            _IsInDesignMode = _HostProcesses.Contains(Process.GetCurrentProcess().ProcessName)
         End If
         Return _IsInDesignMode.Value
      End Function

      Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

      Protected Function ChangePropIfDifferent(Of T)(value As T, ByRef backingField As T, ParamArray propNames() As String) As Boolean
         Return ChangePropIfDifferent(value, backingField, propNames.ToArray(Function(s) New PropertyChangedEventArgs(s)))
      End Function
      Protected Function ChangePropIfDifferent(Of T)(value As T, ByRef backingField As T, ParamArray props() As PropertyChangedEventArgs) As Boolean
         If Object.Equals(backingField, value) Then Return False
         backingField = value
         RaisePropChanged(props)
         Return True
      End Function
      Public Sub RaisePropChanged(ParamArray names() As String)
         RaisePropChanged(names.ToArray(Function(name) New PropertyChangedEventArgs(name)))
      End Sub
      Public Sub RaisePropChanged(ParamArray eventArgses() As PropertyChangedEventArgs)
         RaisePropChanged(DirectCast(eventArgses, IEnumerable(Of PropertyChangedEventArgs)))
      End Sub
      Public Sub RaisePropChanged(eventArgses As IEnumerable(Of PropertyChangedEventArgs))
         VerifyPropertyNames(eventArgses)
         For Each e In eventArgses
            RaiseEvent PropertyChanged(Me, e)
         Next
      End Sub
      Public Sub RaiseAllPropChanged()
         RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(Nothing))
      End Sub


      Private Shared _AllProperties As New Dictionary(Of Type, HashSet(Of String))
      <Conditional("DEBUG"), DebuggerStepThrough()> _
      Public Sub VerifyPropertyNames(eventArgses As IEnumerable(Of PropertyChangedEventArgs))
         Dim propNames As HashSet(Of String) = Nothing
         Dim tp = Me.GetType
         If Not _AllProperties.TryGetValue(tp, propNames) Then
            propNames = New HashSet(Of String)(From pd In Me.GetType.GetProperties Select pd.Name)
            _AllProperties.Add(tp, propNames)
         End If
         For Each pe In eventArgses
            If Not propNames.Contains(pe.PropertyName) Then Throw New Exception("trying to raise an unknown PropertyChanged")
         Next
      End Sub

      ''' <summary>Überschreibungen werden auch von ICloneable.Clone() aufgerufen</summary>
      Public Overridable Sub CopyTo(other As Object)
      End Sub
      Public Function Clone() As Object Implements ICloneable.Clone
         Dim cl = MemberwiseClone()
         CopyTo(cl)
         Return cl
      End Function
   End Class

End Namespace
